@@ -39,14 +39,14 @@ def l2norm(t):
 def plain_cosine_sim_attention(
     q: TensorType['b', 'h', 'i', 'd'],
     k: TensorType['b', 'h', 'j', 'd'],
-    v: TensorType['b', 'h', 'j', 'd'],
+    v: TensorType['b', 'h', 'j', 'e'],
     mask: Optional[TensorType['b', 'j']] = None,
     attn_bias: Optional[TensorType['h', 'i', 'j']] = None,
     scale = 10,
     causal = False,
     l2norm_qk = True
 
-) -> TensorType['b', 'h', 'i', 'd']:
+) -> TensorType['b', 'h', 'i', 'e']:
     assert not (causal and exists(mask)), 'mask should not be supplied if causality is needed'
 
     if l2norm_qk:
@@ -80,17 +80,10 @@ class FlashCosineSimAttention(Function):
         mask,
         attn_bias,
         scale,
-        causal,
-        backward_row_tile_size,
-        backward_col_tile_size,
-        backward_row_tiles,
-        backward_col_tiles
+        causal
     ):
-        assert backward_col_tiles == 1
         assert v.shape[-1] == 64, 'value dimension is fixed at 64 for now'
 
-        assert backward_row_tile_size == backward_col_tile_size
-
         batch, heads, seq, _, dim, device, dtype = *q.shape, v.shape[-1], q.device, q.dtype
 
         mask = default(mask, lambda: torch.ones(q.shape[0], 0, device = q.device, dtype = torch.bool))
@@ -117,11 +110,7 @@ class FlashCosineSimAttention(Function):
 
         ctx.params = (
             scale,
-            causal,
-            backward_row_tile_size,
-            backward_col_tile_size,
-            backward_row_tiles,
-            backward_col_tiles
+            causal
         )
 
         return o
@@ -136,27 +125,17 @@ class FlashCosineSimAttention(Function):
 
         (
             scale,
-            causal,
-            backward_row_tile_size,
-            backward_col_tile_size,
-            backward_row_tiles,
-            backward_col_tiles
+            causal
         ) = ctx.params
 
-        db = torch.zeros((heads, src_seq, tgt_seq), device = device, dtype = dtype) if attn_bias.requires_grad else torch.zeros((heads, 0, 0), device = device, dtype = dtype)
-
-        dq, dk, dv = backward(
+        dq, dk, dv, db = backward(
             do, o, l,
             q, k, v,
-            db,
             mask,
             attn_bias,
             scale,
             causal,
-            backward_row_tile_size,
-            backward_col_tile_size,
-            backward_row_tiles,
-            backward_col_tiles
+            attn_bias.requires_grad
         )
 
         db = db if attn_bias.requires_grad else None
@@ -168,7 +147,7 @@ class FlashCosineSimAttention(Function):
 def flash_cosine_sim_attention(
     q: TensorType['b', 'h', 'i', 'd'],
     k: TensorType['b', 'h', 'j', 'd'],
-    v: TensorType['b', 'h', 'j', 'd'],
+    v: TensorType['b', 'h', 'j', 'e'],
     mask: Optional[TensorType['b', 'j']] = None,
     attn_bias: Optional[TensorType['h', 'i', 'j']] = None,
     scale = 10,
@@ -178,7 +157,7 @@ def flash_cosine_sim_attention(
     backward_col_tile_size = 16,
     backward_row_tiles = 1,
     backward_col_tiles = 1,
-) -> TensorType['b', 'h', 'i', 'd']:
+) -> TensorType['b', 'h', 'i', 'e']:
 
     assert not (causal and exists(mask)), 'mask should not be supplied if causality is needed'
 
@@ -190,11 +169,7 @@ def flash_cosine_sim_attention(
         mask,
         attn_bias,
         scale,
-        causal,
-        backward_row_tile_size,
-        backward_col_tile_size,
-        backward_row_tiles,
-        backward_col_tiles
+        causal
     )
 
     return o
